@@ -25,33 +25,20 @@
 #include "gnome-wp-xml.h"
 #include "wp-cellrenderer.h"
 #include <glib/gi18n.h>
+#include <gio/gio.h>
 #include <string.h>
 #include <gconf/gconf-client.h>
 #include <libgnomeui/gnome-thumbnail.h>
 #define GNOME_DESKTOP_USE_UNSTABLE_API
 #include <libgnomeui/gnome-bg.h>
 
-typedef enum {
-  GNOME_WP_SHADE_TYPE_SOLID,
-  GNOME_WP_SHADE_TYPE_HORIZ,
-  GNOME_WP_SHADE_TYPE_VERT
-} GnomeWPShadeType;
-
-typedef enum {
-  GNOME_WP_SCALE_TYPE_CENTERED,
-  GNOME_WP_SCALE_TYPE_STRETCHED,
-  GNOME_WP_SCALE_TYPE_SCALED,
-  GNOME_WP_SCALE_TYPE_ZOOM,
-  GNOME_WP_SCALE_TYPE_TILED
-} GnomeWPScaleType;
-
 enum {
   TARGET_URI_LIST,
   TARGET_BGIMAGE
 };
 
 static const GtkTargetEntry drop_types[] = {
-  {"text/uri-list", 0, TARGET_URI_LIST},
+  { "text/uri-list", 0, TARGET_URI_LIST },
   { "property/bgimage", 0, TARGET_BGIMAGE }
 };
 
@@ -248,57 +235,23 @@ wp_add_images (AppearanceData *data,
 
 static void
 wp_option_menu_set (AppearanceData *data,
-                    const gchar *value,
+                    int value,
                     gboolean shade_type)
 {
   if (shade_type)
   {
-    if (!strcmp (value, "horizontal-gradient"))
-    {
-      gtk_combo_box_set_active (GTK_COMBO_BOX (data->wp_color_menu),
-                                GNOME_WP_SHADE_TYPE_HORIZ);
-      gtk_widget_show (data->wp_scpicker);
-    }
-    else if (!strcmp (value, "vertical-gradient"))
-    {
-      gtk_combo_box_set_active (GTK_COMBO_BOX (data->wp_color_menu),
-                                GNOME_WP_SHADE_TYPE_VERT);
-      gtk_widget_show (data->wp_scpicker);
-    }
-    else
-    {
-      gtk_combo_box_set_active (GTK_COMBO_BOX (data->wp_color_menu),
-                                GNOME_WP_SHADE_TYPE_SOLID);
+    gtk_combo_box_set_active (GTK_COMBO_BOX (data->wp_color_menu),
+                              value);
+
+    if (value == GNOME_BG_COLOR_SOLID)
       gtk_widget_hide (data->wp_scpicker);
-    }
+    else
+      gtk_widget_show (data->wp_scpicker);
   }
   else
   {
-    if (!strcmp (value, "centered"))
-    {
-      gtk_combo_box_set_active (GTK_COMBO_BOX (data->wp_style_menu),
-                                GNOME_WP_SCALE_TYPE_CENTERED);
-    }
-    else if (!strcmp (value, "stretched"))
-    {
-      gtk_combo_box_set_active (GTK_COMBO_BOX (data->wp_style_menu),
-                                GNOME_WP_SCALE_TYPE_STRETCHED);
-    }
-    else if (!strcmp (value, "scaled"))
-    {
-      gtk_combo_box_set_active (GTK_COMBO_BOX (data->wp_style_menu),
-                                GNOME_WP_SCALE_TYPE_SCALED);
-    }
-    else if (!strcmp (value, "zoom"))
-    {
-      gtk_combo_box_set_active (GTK_COMBO_BOX (data->wp_style_menu),
-                                GNOME_WP_SCALE_TYPE_ZOOM);
-    }
-    else if (!strcmp (value, "none"))
-    {
-      gtk_combo_box_set_active (GTK_COMBO_BOX (data->wp_style_menu),
-                                GNOME_WP_SCALE_TYPE_TILED);
-    }
+    gtk_combo_box_set_active (GTK_COMBO_BOX (data->wp_style_menu),
+                              value);
   }
 }
 
@@ -353,29 +306,7 @@ wp_scale_type_changed (GtkComboBox *combobox,
   if (item == NULL)
     return;
 
-  g_free (item->options);
-
-  switch (gtk_combo_box_get_active (GTK_COMBO_BOX (data->wp_style_menu)))
-  {
-  case GNOME_WP_SCALE_TYPE_CENTERED:
-    item->options = g_strdup ("centered");
-    break;
-  case GNOME_WP_SCALE_TYPE_STRETCHED:
-    item->options = g_strdup ("stretched");
-    break;
-  case GNOME_WP_SCALE_TYPE_SCALED:
-    item->options = g_strdup ("scaled");
-    break;
-  case GNOME_WP_SCALE_TYPE_ZOOM:
-    item->options = g_strdup ("zoom");
-    break;
-  case GNOME_WP_SCALE_TYPE_TILED:
-    item->options = g_strdup ("wallpaper");
-    break;
-  default:
-    item->options = g_strdup ("none");
-    break;
-  }
+  item->options = gtk_combo_box_get_active (GTK_COMBO_BOX (data->wp_style_menu));
 
   pixbuf = gnome_wp_item_get_thumbnail (item, data->thumb_factory);
   gtk_list_store_set (GTK_LIST_STORE (data->wp_model),
@@ -385,7 +316,9 @@ wp_scale_type_changed (GtkComboBox *combobox,
   if (pixbuf != NULL)
     g_object_unref (pixbuf);
 
-  gconf_client_set_string (data->client, WP_OPTIONS_KEY, item->options, NULL);
+  if (gconf_client_key_is_writable (data->client, WP_OPTIONS_KEY, NULL))
+    gconf_client_set_string (data->client, WP_OPTIONS_KEY,
+                             wp_item_option_to_string (item->options), NULL);
 }
 
 static void
@@ -401,24 +334,7 @@ wp_shade_type_changed (GtkWidget *combobox,
   if (item == NULL)
     return;
 
-  g_free (item->shade_type);
-
-  switch (gtk_combo_box_get_active (GTK_COMBO_BOX (data->wp_color_menu)))
-  {
-  case GNOME_WP_SHADE_TYPE_HORIZ:
-    item->shade_type = g_strdup ("horizontal-gradient");
-    gtk_widget_show (data->wp_scpicker);
-    break;
-  case GNOME_WP_SHADE_TYPE_VERT:
-    item->shade_type = g_strdup ("vertical-gradient");
-    gtk_widget_show (data->wp_scpicker);
-    break;
-  case GNOME_WP_SHADE_TYPE_SOLID:
-  default:
-    item->shade_type = g_strdup ("solid");
-    gtk_widget_hide (data->wp_scpicker);
-    break;
-  }
+  item->shade_type = gtk_combo_box_get_active (GTK_COMBO_BOX (data->wp_color_menu));
 
   pixbuf = gnome_wp_item_get_thumbnail (item, data->thumb_factory);
   gtk_list_store_set (GTK_LIST_STORE (data->wp_model), &iter,
@@ -427,8 +343,9 @@ wp_shade_type_changed (GtkWidget *combobox,
   if (pixbuf != NULL)
     g_object_unref (pixbuf);
 
-  gconf_client_set_string (data->client, WP_SHADING_KEY,
-                           item->shade_type, NULL);
+  if (gconf_client_key_is_writable (data->client, WP_SHADING_KEY, NULL))
+    gconf_client_set_string (data->client, WP_SHADING_KEY,
+                             wp_item_shading_to_string (item->shade_type), NULL);
 }
 
 static void
@@ -554,8 +471,7 @@ wp_options_changed (GConfClient *client, guint id,
 
   if (item != NULL)
   {
-    g_free (item->options);
-    item->options = g_strdup (option);
+    item->options = wp_item_string_to_option (option);
     wp_option_menu_set (data, item->options, FALSE);
   }
 }
@@ -573,8 +489,7 @@ wp_shading_changed (GConfClient *client, guint id,
 
   if (item != NULL)
   {
-    g_free (item->shade_type);
-    item->shade_type = g_strdup (gconf_value_get_string (entry->value));
+    item->shade_type = wp_item_string_to_shading (gconf_value_get_string (entry->value));
     wp_option_menu_set (data, item->shade_type, TRUE);
   }
 }
@@ -638,16 +553,18 @@ wp_props_wp_set (AppearanceData *data, GnomeWPItem *item)
       uri = g_filename_to_utf8 (item->filename, -1, NULL, NULL, NULL);
 
     if (uri == NULL) {
-      g_warning ("Failed to convert filename to UTF-8: %s\n", item->filename);
+      g_warning ("Failed to convert filename to UTF-8: %s", item->filename);
     } else {
       gconf_change_set_set_string (cs, WP_FILE_KEY, uri);
       g_free (uri);
     }
 
-    gconf_change_set_set_string (cs, WP_OPTIONS_KEY, item->options);
+    gconf_change_set_set_string (cs, WP_OPTIONS_KEY,
+                                 wp_item_option_to_string (item->options));
   }
 
-  gconf_change_set_set_string (cs, WP_SHADING_KEY, item->shade_type);
+  gconf_change_set_set_string (cs, WP_SHADING_KEY,
+                               wp_item_shading_to_string (item->shade_type));
 
   pcolor = gdk_color_to_string (item->pcolor);
   scolor = gdk_color_to_string (item->scolor);
@@ -722,15 +639,15 @@ wp_drag_received (GtkWidget *widget,
 {
   if (info == TARGET_URI_LIST || info == TARGET_BGIMAGE)
   {
-    GList * uris;
-    GSList * realuris = NULL;
+    GSList *realuris = NULL;
+    gchar **uris;
 
-    uris = gnome_vfs_uri_list_parse ((gchar *) selection_data->data);
-
-    if (uris != NULL && uris->data != NULL)
+    uris = g_uri_list_extract_uris ((gchar *) selection_data->data);
+    if (uris != NULL)
     {
       GdkWindow *window;
       GdkCursor *cursor;
+      gchar **uri;
 
       window = glade_xml_get_widget (data->xml, "appearance_window")->window;
 
@@ -739,16 +656,20 @@ wp_drag_received (GtkWidget *widget,
       gdk_window_set_cursor (window, cursor);
       gdk_cursor_unref (cursor);
 
-      for (; uris != NULL; uris = uris->next)
+      for (uri = uris; *uri; ++uri)
       {
-        realuris = g_slist_append (realuris,
-            g_strdup (gnome_vfs_uri_get_path (uris->data)));
+        GFile *f;
+
+        f = g_file_new_for_uri (*uri);
+        realuris = g_slist_append (realuris, g_file_get_path (f));
+        g_object_unref (f);
       }
 
       wp_add_images (data, realuris);
       gdk_window_set_cursor (window, NULL);
+
+      g_strfreev (uris);
     }
-    gnome_vfs_uri_list_free (uris);
   }
 }
 
@@ -844,16 +765,28 @@ wp_update_preview (GtkFileChooser *chooser,
   if (uri)
   {
     GdkPixbuf *pixbuf = NULL;
-    gchar *mime_type;
+    const gchar *mime_type = NULL;
+    GFile *file;
+    GFileInfo *file_info;
 
-    mime_type = gnome_vfs_get_mime_type (uri);
+    file = g_file_new_for_uri (uri);
+    file_info = g_file_query_info (file,
+                                   G_FILE_ATTRIBUTE_STANDARD_CONTENT_TYPE,
+                                   G_FILE_QUERY_INFO_NONE,
+                                   NULL, NULL);
+    g_object_unref (file);
+
+    if (file_info != NULL)
+    {
+      mime_type = g_file_info_get_content_type (file_info);
+      g_object_unref (file_info);
+    }
 
     if (mime_type)
     {
       pixbuf = gnome_thumbnail_factory_generate_thumbnail (data->thumb_factory,
                                                            uri,
                                                            mime_type);
-      g_free (mime_type);
     }
 
     if (pixbuf != NULL)
@@ -876,8 +809,8 @@ static gboolean
 wp_load_stuffs (void *user_data)
 {
   AppearanceData *data;
-  gchar * imagepath, * style, * uri;
-  GnomeWPItem * item;
+  gchar *imagepath, *uri, *style;
+  GnomeWPItem *item;
 
   data = (AppearanceData *) user_data;
 
@@ -885,8 +818,6 @@ wp_load_stuffs (void *user_data)
   g_hash_table_foreach (data->wp_hash, (GHFunc) wp_props_load_wallpaper,
                         data);
 
-  /*gdk_window_set_cursor (data->window->window, NULL);*/
-
   style = gconf_client_get_string (data->client,
                                    WP_OPTIONS_KEY,
                                    NULL);
@@ -908,22 +839,21 @@ wp_load_stuffs (void *user_data)
 
   item = g_hash_table_lookup (data->wp_hash, imagepath);
 
-  if (item != NULL && strcmp (style, "none") != 0)
+  if (item != NULL)
   {
-    if (item->deleted == TRUE)
+    /* update with the current gconf settings */
+    gnome_wp_item_update (item);
+
+    if (strcmp (style, "none") != 0)
     {
-      item->deleted = FALSE;
-      wp_props_load_wallpaper (item->filename, item, data);
+      if (item->deleted == TRUE)
+      {
+        item->deleted = FALSE;
+        wp_props_load_wallpaper (item->filename, item, data);
+      }
+
+      select_item (data, item, FALSE);
     }
-
-    select_item (data, item, FALSE);
-
-    wp_option_menu_set (data, item->options, FALSE);
-    wp_option_menu_set (data, item->shade_type, TRUE);
-
-    gtk_color_button_set_color (GTK_COLOR_BUTTON (data->wp_pcpicker), item->pcolor);
-    gtk_color_button_set_color (GTK_COLOR_BUTTON (data->wp_scpicker), item->scolor);
-
   }
   else if (strcmp (style, "none") != 0)
   {
@@ -951,7 +881,7 @@ wp_load_stuffs (void *user_data)
     if (!strcmp (style, "none"))
     {
       select_item (data, item, FALSE);
-      wp_option_menu_set (data, style, FALSE);
+      wp_option_menu_set (data, GNOME_BG_PLACEMENT_SCALED, FALSE);
     }
   }
   g_free (imagepath);
@@ -1001,8 +931,6 @@ desktop_init (AppearanceData *data,
 
   data->wp_hash = g_hash_table_new (g_str_hash, g_str_equal);
 
-  gconf_client_add_dir (data->client, WP_KEYBOARD_PATH,
-      GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
   gconf_client_add_dir (data->client, WP_PATH_KEY,
       GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 
